/**
 * A tree control to display contents of an NWN file
 */

#include "FileStructureTree.h"

extern char* MyName;

FileStructureTree::FileStructureTree(wxWindow *parent, const wxWindowID id,
				     const wxPoint& pos, const wxSize& size,
				     long style)
    : wxTreeCtrl(parent, id, pos, size, style) {
    
}

void FileStructureTree::appendListItem(const wxTreeItemId& parent,
				       NWNDataType& item,
				       NWNDataType& parentItem) {
    wxTreeItemId newItem = AppendItem(parent,wxString(item.getReadableName().c_str()),
				      -1,-1,
				      new TreeItemData(item));
    std::map<std::string,NWNDataType>::iterator i = item.listValue.begin();   
    for(;i != item.listValue.end();i++) {
	if(i->second.listValue.size() > 0) {
	    appendListItem(newItem, i->second, item);
	}
    }
}    

wxTreeItemId FileStructureTree::appendListItems(const wxTreeItemId& parent,
					const ITPResource& gic,
					std::string listLabel,
					std::string itemName) {
    wxTreeItemId itemId = AppendItem(parent, wxString(listLabel.c_str()));
    NWNDataType item = gic.getItemByName(itemName.c_str());
    std::map<std::string,NWNDataType>::iterator i = item.listValue.begin();   
    for(;i != item.listValue.end();i++) {
	appendListItem(itemId, i->second, item);
    }
    return itemId;
}

void FileStructureTree::appendListItems(const wxTreeItemId& parent,
					const ITPResource& gic,
					std::string itemName) {
    NWNDataType item = gic.getItemByName(itemName.c_str());
    std::map<std::string,NWNDataType>::iterator i = item.listValue.begin();   
    for(;i != item.listValue.end();i++) {
	appendListItem(parent, i->second, item);
    }
}

void FileStructureTree::treeFromMODFile(MODFile& mod) {
    wxTreeItemId rootId = AddRoot(wxT("Root"));
    wxTreeItemId areasId = AppendItem(rootId, wxString("Areas"));
    MODFile::iterator i = mod.begin();
    for(;i != mod.end();i++) {
	if(mod.getType(i->second) == "are") {
	    printf("found are at %d\n", i->second);
	    ITPResource& area = mod.getITPResource(i->second);
	    wxTreeItemId areaId = AppendItem(areasId, wxString(area.getDescription().c_str()));

	    //ITPResource& gic = mod.getITPResource(area.getResRef(), "gic");

	    ITPResource& git = mod.getITPResource(area.getResRef(), "git");

	    wxTreeItemId creaturesId = appendListItems(areaId,git,"Creatures","Creature List");
	    //appendListItems(creaturesId,git,"Creature List");
	    wxTreeItemId doorsId = appendListItems(areaId,git,"Doors","Door List");
	    //appendListItems(doorsId,git,"Door List");
	    wxTreeItemId encountersId = appendListItems(areaId,git,"Encounters","Encounter List");
	    //appendListItems(encountersId,git,"Encounter List");
	    wxTreeItemId itemsId = appendListItems(areaId,git,"Items","Item List");
	    //appendListItems(itemsId,git,"Item List");
	    wxTreeItemId merchantsId = appendListItems(areaId,git,"Merchants","StoreList");
	    //appendListItems(merchantsId,git,"StoreList");
	    wxTreeItemId placeablesId = appendListItems(areaId,git,"Placeables","Placeable List");
	    //appendListItems(placeablesId,git,"Placeable List");
	    wxTreeItemId soundsId = appendListItems(areaId,git,"Sounds","SoundList");
	    //appendListItems(soundsId,git,"SoundList");
	    wxTreeItemId triggersId = appendListItems(areaId,git,"Triggers","TriggerList");
	    //appendListItems(triggersId,git,"TriggerList");
	    wxTreeItemId waypointsId = appendListItems(areaId,git,"Waypoints","WaypointList");            
	    //appendListItems(waypointsId,git,"WaypointList");
	}
    }
}
