from ERFFile import ERFFile
from GFFFile import GFFStruct
from Area import Area
from Progressor import Progressor
from NeverData import NeverData

class Module(Progressor,NeverData):
    ifoPropList = {
        "Mod_DawnHour":"Integer,0-23",
        "Mod_Description":"CExoLocString,4",
        "Mod_DuskHour":"Integer,0-23",
        "Mod_Entry_Area":"ResRef,ARE",
        "Mod_HakList":"CheckList,HAKs",
        "Mod_Hak":"CheckList,HAK",
        "Mod_Name":"CExoLocString"
        }
    def __init__(self,fname):
        self.erfFile = ERFFile()
        self.erfFile.fromFile(fname)
        ifoEntry = self.erfFile.getEntryByNameAndExtension("module","IFO")
        self.ifoFile = self.erfFile.getEntryContents(ifoEntry)
        
    def getProperty(self,label):
        if label in ifoPropList:
            return [label,
                    self.ifoFile.getRoot().getInterpretedEntry(label),
                    self.ifoPropList[label]]

    def removeProperty(self,label):
        if label in self.ifoPropList:
            (s,t) = self.getTargetStructAndLabel(self.ifoFile,label)
            print 'removing',t
            s.removeEntry(t)
            
    def setProperty(self,label,value):
        if label in self.ifoPropList:
            (s,t) = self.getTargetStructAndLabel(self.ifoFile,label)
            print 'setting',t,value
            if t == 'Mod_HakList':
                converted = []
                for n in value:
                    hak = GFFStruct(8)
                    hak.add('Mod_Hak',n,'CExoString')
                    print 'adding hak',n
                    converted.append(hak)
                value = converted
            s.setInterpretedEntry(t,value)

    def iterateProperties(self):
        pList = self.ifoPropList.keys()
        pList.sort()
        for label in pList:
            (s,t) = self.getTargetStructAndLabel(self.ifoFile,label)
            if t in s.entries:
                yield [label,\
                       s.getInterpretedEntry(t),\
                       self.ifoPropList[label]]

    def getName(self,lang=0):
        return self.ifoFile.getRoot().getInterpretedEntry('Mod_Name').getString()
    
    def getAreaNames(self):
        """Get the names of areas in this ERF."""
        areas = self.erfFile.getEntriesWithExtension('ARE')
        names = []
        for area in areas:
            names.append(area.name)
        return names

    def getAreas(self):
        """Get the areas in this ERF.
        @return a list of Area objects."""
        names = self.getAreaNames() #[:3] #for quicker testing
        print 'areas are ' + `names`
        areas = {}
        c = 1.0
        for n in names:
            self.setProgress((c/len(names))*100.0)
            areas[n] = Area(self.erfFile,n)
            c += 1
        self.setProgress(0)
        return areas

    def getScripts(self):
        """Get the scripts in this ERF.
        @return A list of Script objects."""
        entries = self.erfFile.getEntriesWithExtension('NSS')
        scripts = {}
        for s in entries:
            scripts[s.name] = self.erfFile.getEntryContents(s)
        return scripts

    def saveToReadFile(self):
        self.erfFile.saveToReadFile()

    def toFile(self,fpath):
        self.erfFile.toFile(fpath)

    def setProgressDisplay(self,p):
        self.erfFile.setProgressDisplay(p)

    def getEntriesWithExtension(self,ext):
        return self.erfFile.getEntriesWithExtension(ext)

    def setProgress(self,p):
        self.erfFile.setProgress(p)
