#ifndef _NWN_DATATYPE_H_
#define _NWN_DATATYPE_H_

#include <string>
#include <vector>
#include <sstream>

#include "libnw/itp.h"
#include "config.h"

class NWNDataType {
 public:
    nwn_itp_typecode type;
    union scalarValues {
	unsigned char uint8Value;
	unsigned short uint16Value;
	unsigned int uint32Value;
	unsigned long long uint64Value;
	char int8Value;
	short int16Value;
	int int32Value;
	long long int64Value;
	float floatValue;
	double doubleValue;
    } scalarValue;
    std::string stringValue;
    std::map<std::string,NWNDataType> listValue;
    std::string name;
    bool isStringRef;
    NWNDataType() : type(t_uint64),isStringRef(false) {
	scalarValue.uint64Value = 0;
    }
    NWNDataType(nwn_itp_typecode t) : type(t),isStringRef(false) {
	scalarValue.uint64Value = 0;
    }

    std::string getTag() const {
	if(type != t_list) {
	    return "";
	} else {
	    std::map<std::string,NWNDataType>::const_iterator i;
	    i = listValue.find("Tag"); 
	    if(i != listValue.end() && i->second.stringValue.size() > 0) {
		return i->second.stringValue;
	    }
	    return "";
	}
    }

    std::string getReadableName() const {
	if(name.length() > 0) {
	    return name;
	}
	std::map<std::string,NWNDataType>::const_iterator i;
	i = listValue.find("LocalizedName"); 
	if(i != listValue.end() && i->second.stringValue.size() > 0) {
	    return i->second.stringValue;
	}
	i = listValue.find("LocName"); 
	if(i != listValue.end() && i->second.stringValue.size() > 0) {
	    return i->second.stringValue;
	}
	i = listValue.find("Name"); 
	if(i != listValue.end() && i->second.stringValue.size() > 0) {
	    return i->second.stringValue;
	}
	i = listValue.find("FirstName"); 
	if(i != listValue.end() && i->second.stringValue.size() > 0) {
	    return i->second.stringValue;
	}
	i = listValue.find("Tag"); 
	if(i != listValue.end() && i->second.stringValue.size() > 0) {
	    return i->second.stringValue;
	}
	i = listValue.find("Comment"); 
	if(i != listValue.end() && i->second.stringValue.size() > 0) {
	    return i->second.stringValue;
	}
	if(listValue.size() == 1) {
	    return listValue.begin()->first;
	} else {
	    return "no name found";
	}
    }

    std::string toString() const {
	std::ostringstream ostr;
	switch(type) {
	    case t_uint8:
		ostr <<(int)scalarValue.uint8Value;
		break;
	    case t_uint16:
		ostr <<scalarValue.uint16Value;
		break;
	    case t_uint32:
		ostr <<scalarValue.uint32Value;
		break;
	    case t_uint64:
		ostr <<scalarValue.uint64Value;
		break;
	    case t_int8:
		ostr <<(int)scalarValue.int8Value;
		break;
	    case t_int16:
		ostr <<scalarValue.int16Value;
		break;
	    case t_int32:
		ostr <<scalarValue.int32Value;
		break;
	    case t_int64:			
		ostr <<scalarValue.int64Value;
		break;
	    case t_float:
		ostr <<scalarValue.floatValue;
		break;
	    case t_double:
		ostr <<scalarValue.doubleValue;
		break;
	    case t_resref:
	    case t_locstr:
	    case t_string:
		ostr <<stringValue;
		break;
	    case t_list:
		ostr <<"<list>";
		break;
	    default:
		fprintf(stderr,"neveredit: unhandled type in item %s: %d\n",name.c_str(),type);
		break;
	}
	return ostr.str();
    }
};
#endif
